import requests
from PIL import Image
import io


# Login API
def login_api(email, password, api_url):
    """
    Sends login request to the API.

    :param email: User's email
    :param password: User's password
    :param api_url: Base API URL
    :return: Tuple (status_code, response_data)
    """
    try:
        response = requests.post(
            f"{api_url}/auth/login", json={"email": email, "password": password}
        )
        response.raise_for_status()  # Raise an error for 4xx and 5xx responses
        return response.status_code, response.json()  # Return status and response data
    except requests.exceptions.RequestException as e:
        return None, {"error": str(e)}  # Return error message if request fails


# Register API
def register_api(username, email, phone_number, password, api_url):
    """
    Sends a registration request to the API.

    :param username: User's username
    :param email: User's email
    :param phone_number: User's phone number
    :param password: User's password
    :param api_url: Base API URL
    :return: Tuple (status_code, response_data)
    """
    try:
        response = requests.post(
            f"{api_url}/auth/signup",
            json={
                "username": username,
                "email": email,
                "phone_number": phone_number,
                "password": password,
            },
        )
        response.raise_for_status()  # Raise an error for HTTP errors (4xx, 5xx)
        return response.status_code, response.json()  # Return response data
    except requests.exceptions.RequestException as e:
        return None, {"detail": str(e)}  # Return error message if request fails


# Create Shop
def create_shop_api(name, description, file_path, api_url, access_token):
    """
    Sends a request to create a shop with optional image upload.

    :param name: Name of the shop
    :param description: Shop description
    :param file_path: Path to the shop image (optional)
    :param api_url: Base API URL
    :param access_token: User's access token
    :return: Tuple (status_code, response_data)
    """
    if not access_token:
        return None, {"detail": "Access token not found. Please log in."}

    url = f"{api_url}/shops"
    data = {"name": name, "description": description}
    headers = {"Authorization": f"Bearer {access_token}"}
    files = {}

    if file_path:
        try:
            files["file"] = open(file_path, "rb")
        except Exception as e:
            return None, {"detail": f"Unable to open file: {str(e)}"}

    try:
        response = requests.post(url, data=data, files=files, headers=headers)
        if "file" in files:
            files["file"].close()

        return response.status_code, response.json()
    except requests.exceptions.RequestException as e:
        return None, {"detail": str(e)}


# View Shop
def fetch_shop_details(api_url, token, shop_id=1):
    """
    Fetches details of the shop owned by the logged-in user.

    :param api_url: Base API URL
    :param token: Authorization token
    :param shop_id: ID of the shop (default = 1)
    :return: Dictionary containing shop details or error message
    """
    headers = {"Authorization": f"Bearer {token}"}
    try:
        response = requests.get(f"{api_url}/shops/{shop_id}", headers=headers)
        if response.status_code == 200:
            return response.json()
        else:
            return {"error": "Failed to fetch shop details"}
    except requests.exceptions.RequestException as e:
        return {"error": str(e)}


# Fetch Products
def fetch_shop_products(api_url, token, shop_id):
    """
    Fetches products for a given shop.

    :param api_url: Base API URL
    :param token: Authorization token
    :param shop_id: ID of the shop
    :return: List of products or error message
    """
    headers = {"Authorization": f"Bearer {token}"}
    try:
        response = requests.get(
            f"{api_url}/products?shop_id={shop_id}", headers=headers
        )
        if response.status_code == 200:
            return response.json()
        else:
            return {"error": "Failed to fetch products"}
    except requests.exceptions.RequestException as e:
        return {"error": str(e)}


# Load Image from URL
def load_image_from_url(image_url, size=(150, 150)):
    """
    Loads and resizes an image from a given URL.

    :param image_url: URL of the image
    :param size: Tuple indicating image size
    :return: PIL ImageTk object or None
    """
    try:
        image_response = requests.get(image_url)
        image = Image.open(io.BytesIO(image_response.content))
        image = image.resize(size)
        return image
    except Exception:
        return None
